/* -*- Mode: C; tab-width: 3; indent-tabs-mode: t; c-basic-offset: 3 -*- */
/*
  decode.c
  heraia - an hexadecimal file editor and analyser based on ghex
 
  (C) Copyright 2005 Olivier Delhomme
  e-mail : heraia@delhomme.org
  URL    : http://heraia.tuxfamily.org
 
  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2, or  (at your option) 
  any later version.
 
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY;  without even the implied warranty of
  MERCHANTABILITY  or  FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
 
  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA. */

#include <gtk/gtk.h>
#include <string.h>

#include "decode.h"


/*
   general purpose of this function is to take a 4 byte data stream
   and convert it as if it is a dos date. If it is not, the result
   may be funny !
   We do asume that both *data and *result contains correct values
   and has enough space to store any result.
   data : 4 guchars
   result : at least 21 gchars
*/
gint decode_dos_date( guchar *data, gchar *result )
{
	guint16 year, month, day, hour, minutes, seconds;

	if ((data == NULL) || (result == NULL))
		return FALSE;
	else
		{
			year = (data[0] >> 1) + 1980;
			month = ((data[0] & 0x01) << 3) + (data[1] >> 5);
			day = data[1] & 0x1F;
			hour = (data[2] & 0xF8) >> 3;
			minutes = ((data[2] & 0x07) << 3) + ((data[3] & 0xE0) >> 5);
			seconds = (data[3] & 0x1F) << 1;
			sprintf(result, "%02d/%02d/%04d - %02d:%02d:%02d", day, month, year, 
					  hour, minutes, seconds);
			return TRUE;
		}
}


/*
   general purpose of this function is to take a 8 byte data stream
   and convert it as if it is a filetime date. If it is not, the result
   may be funny !
   We do asume that both *data and *result contains correct values
   and has enough space to store any result.
   data : 8 guchars
   result : at least 21 gchars
*/
gint decode_filetime_date( guchar *data, gchar *result )
{
	unsigned long long year=0, month=0, day=0, hour=0, minutes=0, seconds=0;
	unsigned long long total=0;
	unsigned long long modulus=0;
	unsigned long long reste=0;
	/* number of 100 nanoseconds between 1/1/1601 and 1/1/1970 */ 
	unsigned long long magic_number_date = 116444736000000000;
	/* nanoseconds in one year divided by 10000000 */
	unsigned long long ns_one_year = 31556926; 
	if ((data == NULL) || (result == NULL))
		return FALSE;
	else
		{
			bcopy (data, &total, 8 * sizeof (guchar));
			year = (total / ns_one_year) / 10000000 ;
			reste = total - (year * ns_one_year * 10000000);
			/*
			total = reste;
			month = (total / (ns_one_year/12)) / 10000000;
			reste = total - (month * (ns_one_year/12) * 10000000);

			total = reste;
			day = (total / (ns_one_year/365)) / 10000000;
			reste = total - (day * (ns_one_year/365) * 10000000);
			
			total = reste;
			hour =  (total / (ns_one_year/365/24))  / 10000000;
			reste = total - (hour * (ns_one_year/365/24) * 10000000);

			total = reste;
			minutes = (total / (ns_one_year/365/24/60)) / 10000000;
			reste = total - (minutes * (ns_one_year/365/24/60) * 10000000);
			
			reste = reste / 10000000;
			*/
			year = (total/10000000) / ns_one_year ;
			reste = (total/10000000) - (year * ns_one_year);
			month = reste / (ns_one_year/12);
			reste = reste - (month * (ns_one_year/12));

			day = reste / (ns_one_year/365);
			
			hour = ((total/10000000) % 86400) / 3600;
			minutes = ((total/10000000) % 3600) / 60;
			reste = (total % 600000000) / 10000000;

			day += 1;
			month += 1;
			year += 1601;
			sprintf(result, "%02llu/%02llu/%llu - %02llu:%02llu:%02llu", day, month, year, hour, minutes, reste);
			return TRUE;
		}
}
