/* -*- Mode: C; tab-width: 4; indent-tabs-mode: t; c-basic-offset: 4 -*- */
/*
  heraia.c
  heraia - an hexadecimal file editor and analyser based on ghex
 
  (C) Copyright 2005 Olivier Delhomme
  e-mail : heraia@delhomme.org
  URL    : http://heraia.tuxfamily.org
 
  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2, or  (at your option) 
  any later version.
 
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY;  without even the implied warranty of
  MERCHANTABILITY  or  FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
 
  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA. */

#include <gtk/gtk.h>
#include <gtkhex/gtkhex.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "data_interpretor.h"
#include "heraia.h"


static void version()
{
	fprintf (stdout, "heraia, %s - %s - Version %s - License %s\n", HERAIA_AUTHORS,
			 HERAIA_DATE, HERAIA_VERSION, HERAIA_LICENSE);
}

static int usage(int status)
{
	if (status == 0)
		{
			fprintf (stderr, 
					 "Try `heraia --help' for more information.\n");
			return FALSE;
		}
	else
		{
			version();
			fprintf(stdout, "\nheraia is a simple hexadecimal file editor and \
file analyser");
			fprintf(stdout, "\nUsage :\n\
  heraia [options] filename\n");
			fprintf(stdout, "\nOptions :\n\
  -h, --help\tThis help.\n\
  -v, --version\tProgram version information.\n");
			return TRUE;
		}
	
}

static gboolean delete_main_window_event( GtkWidget *widget, GdkEvent  *event,
										  gpointer   data )
{
	gtk_widget_destroy (widget);
    return TRUE;
}

static void destroy_main_window( GtkWidget *widget,
								 gpointer   data )
{
    gtk_main_quit ();
}


int main (int argc, char ** argv) 
{  
	Options opt; /* A strcuture to manage the command line options  */
	int c = 0;
	FILE *fp;    /* A file pointer to test the file we want to open */

	gboolean exit_value = TRUE;
	HexDocument *doc;
	GtkWidget *window, *hexwidget;
	Data_Window_Struct *DW;

	opt.filename = NULL; /* At first we do not have any filename */

	while ((c = getopt_long (argc, argv, "vh", long_options, NULL)) != -1)
		{
			switch (c)
				{
				case 0:			/* long options handler */
					break;
	  
				case 'v':
					version();
					break;
	 
				case 'h':
					exit_value = usage(1);
					opt.usage = TRUE;
					break;

				default:
					exit_value = usage(0);
					opt.usage = TRUE;
				}
		}

	if (optind < argc)
		{
			opt.filename = (char*) malloc (sizeof(char) * strlen(argv[optind]) + 1);
			strcpy(opt.filename, argv[optind]);
		}
	else
		{
			if (opt.usage != TRUE)
				{
					exit_value = usage (0);
					opt.usage = TRUE;
				}
		}

	if (opt.usage != TRUE)
		{
			if ((fp = fopen(opt.filename, "r")) != NULL)
				{
					fclose(fp);
			
					/* init of gtk and new window */
					exit_value = gtk_init_check (&argc, &argv);
					window = gtk_window_new(GTK_WINDOW_TOPLEVEL);

					/* signaux (on pose des couteurs) */
					g_signal_connect (G_OBJECT (window), "delete_event", 
									  G_CALLBACK (delete_main_window_event), NULL);
					g_signal_connect (G_OBJECT (window), "destroy", 
									  G_CALLBACK (destroy_main_window), NULL);
 
					doc = hex_document_new_from_file (opt.filename);
					hexwidget = gtk_hex_new (doc);
					gtk_container_add(GTK_CONTAINER(window), hexwidget);
					
					DW = (Data_Window_Struct *) g_malloc0 (sizeof(DW));
					data_interpret (hexwidget, DW); 

					gtk_widget_show_all  (window);

					gtk_main ();
					exit_value = TRUE;
				}
			else
				{
					fprintf (stdout, "Error the file could not be opened : %s\n", opt.filename);
					exit_value = FALSE;
				}
		}

	return !exit_value; /* Apparently gtk TRUE and FALSE are inverted compared to bash ! */
}
